import 'app_exception.dart';

class AppEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  void invoke([In? data]) {
    for (var callback in _callbacks) {
      callback.call(data);
    }
  }
}

class AppMonoEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    if (_callbacks.isEmpty) {
      _callbacks.add(callback);
    } else {
      throw AppException("AppDirectEvent already has a subscriber.");
    }
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  Out invoke([In? data]) {
    if (_callbacks.isEmpty) {
      throw AppException("AppDirectEvent was invoked and has no subscriber.");
    }
    return _callbacks.first.call(data);
  }
}

class AppListEvent<In, Out> {
  final List<Out Function(In? data)> _callbacks = [];

  void subscribe(Out Function(In? data) callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Out Function(In? data) callback) {
    _callbacks.remove(callback);
  }

  List<Out> invoke([In? data, List<Out>? list]) {
    list ??= <Out>[];
    for (var callback in _callbacks) {
      var out = callback.call(data);
      list.add(out);
    }
    return list;
  }
}

class Event {
  final List<Function> _callbacks = [];

  void subscribe(Function callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Function callback) {
    _callbacks.remove(callback);
  }

  void invoke([dynamic data]) {
    for (var callback in _callbacks) {
      if (data != null) {
        callback.call(data);
      } else {
        callback.call();
      }
    }
  }
}

class SetStateEvent<T> {
  final List<Function> _callbacks = [];

  void subscribe(Function callback) {
    _callbacks.add(callback);
  }

  void unsubscribe(Function callback) {
    _callbacks.remove(callback);
  }

  void invoke(T data) {
    for (var callback in _callbacks) {
      callback.call(data);
    }
  }
}

class GetStateEvent<T> {
  final List<Function> _callbacks = [];

  void subscribe(T Function() callback) {
    if (_callbacks.isEmpty) {
      _callbacks.add(callback);
    } else {
      throw AppException("GetStateEvent already has a subscriber.");
    }
  }

  void unsubscribe(T Function() callback) {
    _callbacks.remove(callback);
  }

  T invoke() {
    if (_callbacks.isEmpty) {
      throw AppException("GetStateEvent was invoked and has no subscriber.");
    }
    return _callbacks.first.call();
  }
}
