import 'package:badges/badges.dart';
import 'package:flutter/material.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:flutter_icons/flutter_icons.dart';
import 'package:modular_system_core/modular_system_core.dart';

class EntityTableAppBar<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableAppBar({
    required this.viewModel,
    required this.appViewModel,
    required this.updateTableFuture,
  });

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  final Future updateTableFuture;

  @override
  State<EntityTableAppBar> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableAppBarState<T>(
      appViewModel: appViewModel,
      viewModel: viewModel,
    );
  }
}

class _EntityTableAppBarState<T> extends State<EntityTableAppBar> {
  _EntityTableAppBarState({
    required this.viewModel,
    required this.appViewModel,
  }) {
    subscribeToEvents();
  }

  AppViewModel appViewModel;
  EntityTableViewModel<T> viewModel;
  bool isSelectorOn = false;
  late bool showSelectorActions = true;
  late int filtersCount =
      viewModel.state.search.filters.where((x) => x.isActive).length;
  late Future updateFuture = viewModel.state.updateFuture;

  void subscribeToEvents() {
    viewModel.events.update.subscribe(onUpdate);
    viewModel.events.searchFiltersChanged.subscribe(onSearchFiltersChanged);
  }

  void unsubscribeToEvents() {
    viewModel.events.update.unsubscribe(onUpdate);
    viewModel.events.searchFiltersChanged.unsubscribe(onSearchFiltersChanged);
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  // !try adding an app bar inside the bottom section of the sliver app bar!
  @override
  Widget build(BuildContext context) {
    return SliverAppBar(
      pinned: true,
      elevation: 10,
      backgroundColor: appViewModel.colorScheme.background,
      title: TextField(
        controller: viewModel.state.searchbarController,
        style: appViewModel.backgroundTextStyle(AppTextType.subtitle),
        decoration: InputDecoration(
          hintText: "Pesquisar ...",
          hintStyle: appViewModel.backgroundTextStyle(AppTextType.subtitle),
          focusedBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: appViewModel.colorScheme.onPrimary,
            ),
          ),
          border: InputBorder.none,
        ),
        onChanged: onTextFieldChanged,
      ),
      leading: IconButton(
        icon: Icon(
          Icons.search,
          color: appViewModel.colorScheme.onPrimary,
          size: appViewModel.textSize.subtitle,
        ),
        onPressed: () {
          viewModel.events.update.invoke();
        },
      ),
      actions: [
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: appViewModel.textSize.subtitle * 0.5,
          ),
          icon: Icon(
            Icons.clear,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: () {
            viewModel.state.searchbarController.text = "";
            clearTextFilter();
          },
        ),
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: appViewModel.textSize.subtitle * 0.5,
          ),
          icon: Icon(
            Icons.post_add,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: viewModel.events.showForm.invoke,
        ),
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: appViewModel.textSize.subtitle * 0.5,
          ),
          icon: Icon(
            MaterialCommunityIcons.restore,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: viewModel.events.update.invoke,
        ),
      ],
      bottom: AppBar(
        elevation: 0,
        backgroundColor: appViewModel.colorScheme.background,
        title: Container(
          alignment: Alignment.center,
          //color: Colors.green,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Row(
                children: [
                  Badge(
                    position: BadgePosition.topEnd(),
                    badgeColor: appViewModel.colorScheme.tertiary,
                    badgeContent: Text(
                      filtersCount.toString(),
                      style: appViewModel.tertiaryTextStyle(
                        AppTextType.label,
                      ),
                    ),
                    child: IconButton(
                      icon: Icon(
                        Icons.filter_list,
                        color: appViewModel.colorScheme.onBackground,
                        size: appViewModel.textSize.subtitle,
                      ),
                      onPressed: () {
                        appViewModel.appEvents.bottomSheetEvent.invoke(viewModel
                            .properties
                            .filtersSheetBuilder(viewModel));
                      },
                    ),
                  ),
                  viewModel.config.hasAppbarActions
                      ? VerticalDivider(
                          color: appViewModel.colorScheme.onBackground,
                        )
                      : Container(),
                  viewModel.config.hasAppbarActions
                      ? IconButton(
                          icon: Icon(
                            viewModel.state.extraOptionsOn
                                ? Icons.expand_less
                                : Icons.expand_more,
                            color: viewModel.state.extraOptionsOn
                                ? appViewModel.colorScheme.tertiary
                                : appViewModel.colorScheme.onBackground,
                            size: appViewModel.textSize.subtitle,
                          ),
                          onPressed: () {
                            setState(() {
                              viewModel.state.extraOptionsOn =
                                  !viewModel.state.extraOptionsOn;
                              viewModel.events.showExtraOptionsEvent.invoke();
                            });
                          },
                        )
                      : Container(),
                  VerticalDivider(
                    color: appViewModel.colorScheme.onBackground,
                  ),
                  IconButton(
                    icon: Icon(
                      Icons.check_box,
                      color: viewModel.state.selectorOn
                          ? appViewModel.colorScheme.tertiary
                          : appViewModel.colorScheme.onBackground,
                      size: appViewModel.textSize.subtitle,
                    ),
                    onPressed: () {
                      setState(() {
                        viewModel.events.setSelectorMode.invoke(
                          !viewModel.state.selectorOn,
                        );
                      });
                    },
                  ),
                  VerticalDivider(
                    color: appViewModel.colorScheme.onBackground,
                  ),
                  getSelectorDropdown(),
                  VerticalDivider(
                    color: appViewModel.colorScheme.onBackground,
                  ),
                  getAppbarActions(),
                  FutureBuilder(
                    future: updateFuture,
                    builder: (context, snapshot) {
                      if (snapshot.connectionState == ConnectionState.done &&
                          !snapshot.hasError) {
                        return Row(
                          children: [
                            Padding(
                              padding: const EdgeInsets.symmetric(
                                horizontal: 5,
                              ),
                              child: Icon(
                                MaterialCommunityIcons.table_settings,
                                color: appViewModel.colorScheme.onBackground,
                                size: appViewModel.textSize.subtitle,
                              ),
                            ),
                            Text(
                              "${getPagination()} Total: ${viewModel.state.paginationOut.total}",
                              style: appViewModel
                                  .backgroundTextStyle(AppTextType.subtitle),
                            )
                          ],
                        );
                      } else if (snapshot.hasError) {
                        return Row(
                          children: [
                            Padding(
                              padding: const EdgeInsets.symmetric(
                                horizontal: 5,
                              ),
                              child: Icon(
                                MaterialCommunityIcons.alert,
                                color: appViewModel.colorScheme.onBackground,
                                size: appViewModel.textSize.subtitle,
                              ),
                            ),
                            Text(
                              "Erro",
                              style: appViewModel
                                  .errorTextStyle(AppTextType.subtitle),
                            )
                          ],
                        );
                      } else {
                        return Row(
                          children: [
                            Padding(
                              padding: const EdgeInsets.symmetric(
                                horizontal: 5,
                              ),
                              child: Icon(
                                MaterialCommunityIcons.database_refresh,
                                color: appViewModel.colorScheme.onBackground,
                                size: appViewModel.textSize.subtitle,
                              ),
                            ),
                            const VerticalDivider(),
                            Text(
                              "Carregando...",
                              style: appViewModel
                                  .backgroundTextStyle(AppTextType.subtitle),
                            )
                          ],
                        );
                      }
                    },
                  ),
                ],
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  FutureBuilder(
                    future: updateFuture,
                    builder: (context, snapshot) {
                      if (snapshot.connectionState == ConnectionState.done &&
                          !snapshot.hasError) {
                        return Container(
                          margin: const EdgeInsets.symmetric(
                            horizontal: 25,
                          ),
                          child: Row(
                            children: [
                              IconButton(
                                iconSize: appViewModel.textSize.subtitle,
                                mouseCursor: previousPageExists()
                                    ? null
                                    : SystemMouseCursors.basic,
                                splashRadius: previousPageExists() ? null : 1,
                                color: previousPageExists()
                                    ? appViewModel.colorScheme.tertiary
                                    : appViewModel.colorScheme.onBackground,
                                padding: const EdgeInsets.symmetric(
                                  horizontal: 15,
                                ),
                                icon: const Icon(
                                  Icons.navigate_before,
                                ),
                                onPressed: () {
                                  if (previousPageExists()) {
                                    viewModel.state.search.pagination
                                        .offset = viewModel
                                            .state.search.pagination.offset -
                                        viewModel.state.search.pagination.limit;
                                    viewModel.events.update.invoke();
                                  }
                                },
                              ),
                              IconButton(
                                iconSize: appViewModel.textSize.subtitle,
                                mouseCursor: nextPageExists()
                                    ? null
                                    : SystemMouseCursors.basic,
                                splashRadius: nextPageExists() ? null : 1,
                                color: nextPageExists()
                                    ? appViewModel.colorScheme.tertiary
                                    : appViewModel.colorScheme.onBackground,
                                padding: const EdgeInsets.symmetric(
                                  horizontal: 15,
                                ),
                                icon: const Icon(
                                  Icons.navigate_next,
                                ),
                                onPressed: () {
                                  if (nextPageExists()) {
                                    viewModel.state.search.pagination
                                        .offset = viewModel
                                            .state.search.pagination.offset +
                                        viewModel.state.search.pagination.limit;
                                    viewModel.events.update.invoke();
                                  }
                                },
                              ),
                            ],
                          ),
                        );
                      } else if (snapshot.hasError) {
                        return Text(
                          "ERRO",
                          style:
                              appViewModel.errorTextStyle(AppTextType.subtitle),
                        );
                      } else {
                        return Container();
                      }
                    },
                  ),
                  DropdownButton(
                    value: viewModel.state.search.pagination.limit,
                    focusColor: Colors.transparent,
                    dropdownColor: appViewModel.colorScheme.primary,
                    iconEnabledColor: appViewModel.colorScheme.onBackground,
                    underline: SizedBox(
                      height: 1.5,
                      child: Container(
                        color: appViewModel.colorScheme.onBackground,
                      ),
                    ),
                    items: <DropdownMenuItem<int>>[
                      DropdownMenuItem(
                        value: 10,
                        child: Text(
                          "10",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 30,
                        child: Text(
                          "30",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 50,
                        child: Text(
                          "50",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 100,
                        child: Text(
                          "100",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                    ],
                    onChanged: (limit) {
                      setState(() {
                        viewModel.state.search.pagination.limit = limit as int;
                        viewModel.events.update.invoke();
                      });
                    },
                  ),
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget getSelectorDropdown() {
    if (!showSelectorActions) {
      return Container();
    }

    return PopupMenuButton(
      icon: Icon(
        Icons.more_vert,
        color: appViewModel.colorScheme.onBackground,
        size: appViewModel.textSize.subtitle,
      ),
      itemBuilder: (context) {
        var items = <PopupMenuItem<int>>[];
        var actions = viewModel.properties.selectorActionsBuilder!(viewModel);
        int index;
        for (index = 0; index < actions.length; index++) {
          var action = actions.elementAt(index);
          var item = PopupMenuItem(
            value: index,
            child: action.key,
            onTap: () => action.value.call(),
          );
          items.add(item);
        }
        items.add(PopupMenuItem(
          value: index,
          child: Row(
            children: [
              Icon(
                Icons.cancel,
                color: appViewModel.colorScheme.onBackground,
                size: appViewModel.textSize.data,
              ),
              const VerticalDivider(),
              Text(
                'Cancelar',
                style: appViewModel.backgroundTextStyle(),
              ),
            ],
          ),
          onTap: () {},
        ));
        return items;
      },
    );
  }

  String getPagination() {
    var pagination = viewModel.state.paginationOut;
    var currentPage = pagination.offset / pagination.limit + 1;
    var totalPages = pagination.total / pagination.limit + 1;
    return "$currentPage de ${totalPages.floor()}";
  }

  bool nextPageExists() {
    return viewModel.state.paginationOut.total >
        (viewModel.state.paginationOut.offset +
            viewModel.state.paginationOut.limit);
  }

  bool previousPageExists() {
    return viewModel.state.paginationOut.offset > 0;
  }

  Widget getAppbarActions() {
    if (viewModel.properties.appbarActions != null &&
        viewModel.properties.appbarActions!.isNotEmpty) {
      var children = <Widget>[];
      for (var item in viewModel.properties.appbarActions!) {
        children.add(
          IconButton(
            icon: Icon(
              item.key,
              color: appViewModel.colorScheme.onBackground,
              size: appViewModel.textSize.subtitle,
            ),
            onPressed: () {
              item.value.call(viewModel);
            },
          ),
        );
      }
      children.add(const VerticalDivider());
      return Row(
        children: children,
      );
    }
    return Container();
  }

  void onTextFieldChanged(String value) {
    viewModel.events.setSearchText.invoke(value);
  }

  void onUpdate() {
    setState(() {
      updateFuture = viewModel.state.updateFuture;
    });
  }

  void onSearchFiltersChanged() {
    setState(() {
      filtersCount =
          viewModel.state.search.filters.where((x) => x.isActive).length;
    });
  }

  void clearTextFilter() {
    viewModel.events.setSearchText.invoke("");
  }
}
