import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';
import 'package:modular_system_core/src/widget/entity_table/table_tile.dart';
export 'viewmodel.dart';
export './app_bar.dart';

class EntityTable<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTable({
    this.searchbarAutoUpdate = false,
    required this.appViewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    this.filtersSheetBuilder,
    this.selectorActionsBuilder,
    this.editorBuilder,
    this.appbarActions,
  });

  // properties
  final bool? searchbarAutoUpdate;
  final AppViewModel appViewModel;
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Duration transitionDuration;
  final List<SingleEntryMap<IconData, Function(EntityTableViewModel vm)>>?
      appbarActions;

  // builders
  final TableItemBuilder<T> itemBuilder;
  final TableWidgetBuilder<T> formBuilder;

  // optional builders
  final AppBottomSheet Function(
    EntityTableViewModel<T> viewModel,
  )? filtersSheetBuilder;

  final List<SingleEntryMap<Widget, Function>> Function(
      EntityTableViewModel<T> viewModel)? selectorActionsBuilder;

  final TableEditorBuilder<T>? editorBuilder;

  @override
  State<EntityTable> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableState<T>(
      appViewModel: appViewModel,
      viewModel: getViewModel(),
    );
  }

  EntityTableViewModel<T> getViewModel() {
    return EntityTableViewModel(
      properties: TableVmProperties(
        update: update,
        itemBuilder: itemBuilder,
        formBuilder: formBuilder,
        transitionDuration: transitionDuration,
        filtersSheetBuilder:
            filtersSheetBuilder ?? (vm) => AppBottomSheet.empityRounded(),
        selectorActionsBuilder: selectorActionsBuilder ?? (vm) => [],
        editorBuilder: editorBuilder,
        appbarActions: appbarActions,
      ),
      config: TableVmConfig(
        hasAppbarActions: false,
        searchbarAutoUpdate: searchbarAutoUpdate ?? false,
      ),
    );
  }
}

class _EntityTableState<T> extends State<EntityTable> {
  _EntityTableState({
    required this.appViewModel,
    required this.viewModel,
  }) {
    subscribeToEvents();
    setUpdateFuture();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  late Widget body = table();

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  void subscribeToEvents() {
    viewModel.events.update.subscribe(onUpdate);
    viewModel.events.showTable.subscribe(onShowTable);
    viewModel.events.showForm.subscribe(onShowForm);
    viewModel.events.showEditor.subscribe(onShowEditor);
    viewModel.events.setSelectorMode.subscribe(onSetSelectorMode);
    viewModel.events.setSearchText.subscribe(onSetSearchText);
  }

  void unsubscribeToEvents() {
    viewModel.events.update.unsubscribe(onUpdate);
    viewModel.events.showTable.unsubscribe(onShowTable);
    viewModel.events.showForm.unsubscribe(onShowForm);
    viewModel.events.showEditor.unsubscribe(onShowEditor);
    viewModel.events.setSelectorMode.unsubscribe(onSetSelectorMode);
    viewModel.events.setSearchText.unsubscribe(onSetSearchText);
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedSwitcher(
      duration: viewModel.properties.transitionDuration,
      child: body,
    );
  }

  Widget table() {
    return Container(
      color: appViewModel.colorScheme.background,
      child: CustomScrollView(
        slivers: [
          EntityTableAppBar(
            viewModel: viewModel,
            appViewModel: appViewModel,
            updateTableFuture: viewModel.state.updateFuture,
          ),
          EntityTableBody(
            appViewModel: appViewModel,
            viewModel: viewModel,
          ),
        ],
      ),
    );
  }

  void setBody(Widget widget) {
    setState(() {
      body = widget;
    });
  }

  void setUpdateFuture() {
    viewModel.state.updateFuture = Future(() async {
      try {
        var map = await viewModel.properties.update(viewModel.state.search);
        viewModel.state.data = map.value;
        viewModel.state.paginationOut = map.key;
        viewModel.events.setData.invoke(map.value);
        viewModel.events.setPagination.invoke(map.key);
      } catch (e) {
        appViewModel.appEvents.errorEvent.invoke(e);
      }
    });
  }

  // Event Handlers
  void onUpdate() {
    setState(() {
      viewModel.state.selectedTiles.clear();
      setUpdateFuture();
    });
  }

  void onShowTable() => setBody(table());

  void onShowForm() {
    setBody(viewModel.properties.formBuilder(
      context,
      viewModel,
    ));
  }

  void onShowEditor(T? data) {
    if (viewModel.properties.editorBuilder != null) {
      setBody(viewModel.properties.editorBuilder!(
        context,
        data!,
        viewModel,
      ));
    }
  }

  void onSetSelectorMode(bool selectorState) {
    viewModel.state.selectorOn = selectorState;
  }

  void onSetSearchText(String value) {
    viewModel.state.searchbarController.text = value;
    viewModel.state.searchbarController.selection = TextSelection.fromPosition(
      TextPosition(
        offset: viewModel.state.searchbarController.text.length,
      ),
    );
    if (viewModel.state.search.filters
        .where((element) => element.name == "text")
        .isEmpty) {
      viewModel.state.search.filters.add(
        WebSearchFilter(
          name: "text",
          type: WebDataType.string,
          isActive: value.isNotEmpty ? true : false,
          value: value,
        ),
      );
    } else {
      viewModel.state.search.filters
          .where((element) => element.name == "text")
          .first
          .value = value;
      viewModel.state.search.filters
          .where((element) => element.name == "text")
          .first
          .isActive = value.isNotEmpty ? true : false;
    }
    viewModel.events.searchFiltersChanged.invoke();
    if (viewModel.config.searchbarAutoUpdate) {
      viewModel.events.update.invoke();
    }
  }
}

class EntityTableBody<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableBody({
    required this.appViewModel,
    required this.viewModel,
  });

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;

  @override
  State<EntityTableBody> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableBodyState(
      appViewModel: appViewModel,
      viewModel: viewModel,
    );
  }
}

class _EntityTableBodyState<T> extends State<EntityTableBody> {
  _EntityTableBodyState({
    required this.appViewModel,
    required this.viewModel,
  }) {
    subscribeToEvents();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  late Future updateFuture = viewModel.state.updateFuture;
  late List<T> data = viewModel.state.data;

  void subscribeToEvents() {
    viewModel.events.update.subscribe(onUpdate);
    viewModel.events.setData.subscribe(onSetData);
  }

  void unsubscribeToEvents() {
    viewModel.events.update.unsubscribe(onUpdate);
    viewModel.events.setData.unsubscribe(onSetData);
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: updateFuture,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.done) {
          if (snapshot.hasError) {
            return SliverToBoxAdapter(
              child: Text(
                "ERRO",
                style: appViewModel.errorTextStyle(AppTextType.subtitle),
              ),
            );
          }
          return SliverList(
            key: UniqueKey(),
            delegate: SliverChildBuilderDelegate(
              (context, index) {
                return TableTile(
                  index: index,
                  appViewModel: appViewModel,
                  viewModel: viewModel,
                  child: viewModel.properties.itemBuilder(
                    context,
                    data[index],
                    viewModel,
                  ),
                );
              },
              childCount: data.length,
            ),
          );
        }
        return SliverToBoxAdapter(
          child: appViewModel.loading(),
        );
      },
    );
  }

  void onUpdate() {
    setState(() {
      updateFuture = viewModel.state.updateFuture;
    });
  }

  void onSetData(List<T> data) {
    setState(() {
      this.data = data;
    });
  }
}
